import { FunctionValueGenerator, ValueGenerator } from "./functions/ValueGenerator";
import { Behavior } from "./behaviors/Behavior";
import { Particle } from "./Particle";
import { ParticleEmitter } from "./ParticleEmitter";
import { EmitterShape, ShapeJSON } from "./EmitterShape";
import { Blending, BufferGeometry, Matrix3, Matrix4, Texture } from "three";
import { ColorGenerator, FunctionColorGenerator } from "./functions/ColorGenerator";
import { FunctionJSON } from "./functions/FunctionJSON";
import { ParticleSystemBatchSettings, RenderMode } from "./ParticleSystemBatch";
import { BatchedParticleRenderer } from "./BatchedParticleRenderer";
export interface BurstParameters {
    time: number;
    count: number;
    cycle: number;
    interval: number;
    probability: number;
}
export interface ParticleSystemParameters {
    autoDestroy?: boolean;
    looping?: boolean;
    duration?: number;
    maxParticle?: number;
    shape?: EmitterShape;
    startLife?: ValueGenerator | FunctionValueGenerator;
    startSpeed?: ValueGenerator | FunctionValueGenerator;
    startRotation?: ValueGenerator | FunctionValueGenerator;
    startSize?: ValueGenerator | FunctionValueGenerator;
    startColor?: ColorGenerator | FunctionColorGenerator;
    emissionOverTime?: ValueGenerator | FunctionValueGenerator;
    emissionOverDistance?: ValueGenerator | FunctionValueGenerator;
    emissionBursts?: Array<BurstParameters>;
    behaviors?: Array<Behavior>;
    instancingGeometry?: BufferGeometry;
    renderMode?: RenderMode;
    speedFactor?: number;
    texture: Texture;
    startTileIndex?: number;
    uTileCount?: number;
    vTileCount?: number;
    renderOrder?: number;
    blending?: Blending;
    worldSpace?: boolean;
}
export interface ParticleSystemJSONParameters {
    autoDestroy: boolean;
    looping: boolean;
    duration: number;
    maxParticle: number;
    shape: ShapeJSON;
    startLife: FunctionJSON;
    startSpeed: FunctionJSON;
    startRotation: FunctionJSON;
    startSize: FunctionJSON;
    startColor: FunctionJSON;
    emissionOverTime: FunctionJSON;
    emissionOverDistance: FunctionJSON;
    emissionBursts?: Array<BurstParameters>;
    instancingGeometry?: any;
    renderMode: number;
    renderOrder?: number;
    speedFactor?: number;
    texture: string;
    startTileIndex: number;
    uTileCount: number;
    vTileCount: number;
    blending: number;
    behaviors: Array<any>;
    worldSpace: boolean;
}
export declare class ParticleSystem {
    autoDestroy: boolean;
    looping: boolean;
    duration: number;
    maxParticle: number;
    startLife: ValueGenerator | FunctionValueGenerator;
    startSpeed: ValueGenerator | FunctionValueGenerator;
    startRotation: ValueGenerator | FunctionValueGenerator;
    startSize: ValueGenerator | FunctionValueGenerator;
    startColor: ColorGenerator | FunctionColorGenerator;
    startTileIndex: number;
    emissionOverTime: ValueGenerator | FunctionValueGenerator;
    emissionOverDistance: ValueGenerator | FunctionValueGenerator;
    emissionBursts: Array<BurstParameters>;
    worldSpace: boolean;
    speedFactor: number;
    particleNum: number;
    private burstIndex;
    private burstWaveIndex;
    private time;
    paused: boolean;
    private waitEmiting;
    private emitEnded;
    private markForDestroy;
    behaviors: Array<Behavior>;
    particles: Array<Particle>;
    emitterShape: EmitterShape;
    emitter: ParticleEmitter;
    rendererSettings: ParticleSystemBatchSettings;
    renderer: BatchedParticleRenderer;
    neededToUpdateRender: boolean;
    get texture(): Texture;
    set texture(texture: Texture);
    get uTileCount(): number;
    set uTileCount(u: number);
    get vTileCount(): number;
    set vTileCount(v: number);
    get renderMode(): RenderMode;
    set renderMode(renderMode: RenderMode);
    get renderOrder(): number;
    set renderOrder(renderOrder: number);
    get blending(): Blending;
    set blending(blending: Blending);
    constructor(renderer: BatchedParticleRenderer, parameters: ParticleSystemParameters);
    pause(): void;
    play(): void;
    spawn(count: number): void;
    oldWorldMatrix: Matrix4;
    normalMatrix: Matrix3;
    endEmit(): void;
    dispose(): void;
    restart(): void;
    update(delta: number): void;
    toJSON(meta: any): ParticleSystemJSONParameters;
    static fromJSON(json: ParticleSystemJSONParameters, textures: {
        [a: string]: Texture;
    }, renderer: BatchedParticleRenderer): ParticleSystem;
    addBehavior(behavior: Behavior): void;
    getRendererSettings(): ParticleSystemBatchSettings;
    clone(): ParticleSystem;
}
